"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBuildVersionName = exports.validateAppConfigForRemoteVersionSource = exports.validateBuildProfileVersionSettings = exports.ensureVersionSourceIsRemoteAsync = void 0;
const tslib_1 = require("tslib");
const eas_build_job_1 = require("@expo/eas-build-job");
const eas_json_1 = require("@expo/eas-json");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const log_1 = tslib_1.__importDefault(require("../log"));
const prompts_1 = require("../prompts");
async function ensureVersionSourceIsRemoteAsync(easJsonAccessor, { nonInteractive }) {
    const easJsonCliConfig = await eas_json_1.EasJsonUtils.getCliConfigAsync(easJsonAccessor);
    if (easJsonCliConfig?.appVersionSource === eas_json_1.AppVersionSource.REMOTE) {
        return;
    }
    if (nonInteractive) {
        throw new Error(`This project is not configured for using remote version source. Add ${chalk_1.default.bold('{"cli": { "appVersionSource": "remote" }}')} in eas.json or re-run this command without "--non-interactive" flag.`);
    }
    log_1.default.log(`The app version source defines whether the app version is stored locally in your project source (e.g. in app.json, Info.plist, or build.gradle) or remotely on EAS servers and only applied to the project at build time. To use this command, you will need to enable the remote version policy by adding  ${chalk_1.default.bold('{"cli": { "appVersionSource": "remote" }}')} in eas.json.`);
    // TODO: add link to docs
    const confirm = await (0, prompts_1.confirmAsync)({
        message: 'Do you want to set app version source to remote now?',
    });
    if (!confirm) {
        throw new Error('Aborting...');
    }
    await easJsonAccessor.readRawJsonAsync();
    easJsonAccessor.patch(easJsonRawObject => {
        easJsonRawObject.cli = { ...easJsonRawObject?.cli, appVersionSource: eas_json_1.AppVersionSource.REMOTE };
        return easJsonRawObject;
    });
    await easJsonAccessor.writeAsync();
    log_1.default.withTick('Updated eas.json');
}
exports.ensureVersionSourceIsRemoteAsync = ensureVersionSourceIsRemoteAsync;
function validateBuildProfileVersionSettings(profileInfo, cliConfig) {
    if (cliConfig?.appVersionSource !== eas_json_1.AppVersionSource.REMOTE) {
        return;
    }
    if (profileInfo.profile.autoIncrement === 'version') {
        throw new Error(`${chalk_1.default.bold('{"autoIncrement": "version"}')} is not supported when app version source is set to remote.`);
    }
}
exports.validateBuildProfileVersionSettings = validateBuildProfileVersionSettings;
function validateAppConfigForRemoteVersionSource(exp, platform) {
    if (typeof exp.runtimeVersion === 'object' && exp.runtimeVersion?.policy === 'nativeVersion') {
        throw new Error(`${chalk_1.default.bold('nativeVersion')} policy for ${chalk_1.default.bold('runtimeVersion')} is currently not supported when version source is set to remote. Switch policy e.g. to ${chalk_1.default.bold('appVersion')} or define version explicitly.`);
    }
    if (platform === eas_build_job_1.Platform.IOS && exp.ios?.buildNumber !== undefined) {
        log_1.default.warn(`${chalk_1.default.bold('ios.buildNumber')} field in app config is ignored when version source is set to remote, but this value will still be in the manifest available via ${chalk_1.default.bold('expo-constants')}. It's recommended to remove this value from app config.`);
    }
    if (platform === eas_build_job_1.Platform.ANDROID && exp.android?.versionCode !== undefined) {
        log_1.default.warn(`${chalk_1.default.bold('android.versionCode')} field in app config is ignored when version source is set to remote, but this value will still be in the manifest available via ${chalk_1.default.bold('expo-constants')}. It's recommended to remove this value from app config.`);
    }
}
exports.validateAppConfigForRemoteVersionSource = validateAppConfigForRemoteVersionSource;
function getBuildVersionName(platform) {
    if (platform === eas_build_job_1.Platform.ANDROID) {
        return 'versionCode';
    }
    else {
        return 'buildNumber';
    }
}
exports.getBuildVersionName = getBuildVersionName;
